/*
              Fecha: 17/05/09
              Autor: Adrin Sergio Buratovich
        Descripcin: Funcionalidad bsica para el ordenamiento
                     por columnas de los items de un objeto GRID.
           Objetivo: Obtener reordenamientos concatenados de las
                     columnas de un objeto GRID partiendo de la
                     columna seleccionada y prosiguiendo con todas
                     las restantes segn un orden establecido.
    Ultima Revisin: 20/05/09 - 17:00:00

  NOTA 01: GRIDSETORDER() no soporta el modo VIRTUAL de la
           definicin del objeto GRID.

           PENDIENTE: VALIDAR ESTRUCTURA DEFINICION VIRTUAL DEL
                      GRID PARA EVITAR FALLO EN

                      this.Item(_nGridCol)

  NOTA 02: GRIDSETORDER() soporta contenidos en tipos C/N/D/L.
           Si se encuentran otros tipos, se suspende el proceso.

           PENDIENTE: EXCLUSION DE CAMPOS DE TIPOS NO SOPORTADOS
                      EN CONTENIDO A ORDENAR


  NOTA 03: GRIDSETORDER() concatena todas las columnas existentes

           PENDIENTE: INTEGRACION DE CRITERIOS DE ORDENAMIENTO
                      CREACION DE FUNCION GRIDCREATEINDEX()


*/

//--------------------------------------------------------------//

FUNCTION GRIDSETORDER( _nGridCol )

*-----------------------
 Local aDataGrid       := {}
 Local nColGrid        := 0
 Local nRowGrid        := 0
 Local nColProces      := 0
*-----------------------
 Local cContOrd        := ""
 Local aValType        := {}
*-----------------------
 Local cGridKey        := ""
 Local aGridOrd        := ""
 Local aGridIndex      := {}
*-----------------------
 Local cFieldSeparator := "|"
 Local cItemSeparator  := "<row>"
 Local nItemOrd        := 0
 Local lTransferError  := .F. 
*-----------------------
 // VARIABLES DE TESTEO A SER ELIMINADAS
 Local lAllCols        := .T. // Modo de procesamiento
 Local lTest           := .F.
 Local aTest           := {}
*-----------------------
 Private uTempData
*-----------------------

 if lTest
  Aadd(aTest,'Inicio del proceso:' + Time())
 endif

 If "N"<>ValType(_nGridCol)
  MsgStop("Error de tipo de dato de orden requerido.","GRIDSETORDER()")
  RETURN(.F.)
 EndIf

 // PENDIENTE: FALTA VALIDAR ESTRUCTURA DE ITEMS EN FUNCION DEL TIPO DE 
 //            DEFINICION VIRTUAL DEL GRID

 If valtype(this.Item(_nGridCol))<>"A"
  MsgStop("Error de tipo de contenidos a ser procesados.","GRIDSETORDER()")
  RETURN(.F.)
 EndIf


 // Determinacin de parmetros del proceso

 // PENDIENTE: INTEGRACION DE CLAVES GENERADAS POR GRIDCREATEINDEX()
 //            Y ACTIVADAS POR GRIDSETINDEX()

 nColGrid  := Len(AClone(this.Item(_nGridCol)))
 nRowGrid  := this.ItemCount
 if lAllCols
  nColProces := nColGrid
 else
  nColProces := _nGridCol
 endif

 // Determinacin de conversores de tipo de dato

 For nCol := 1 To nColProces
  // Se evala el primer item para obtener los tipos de datos de las columnas
  vValType := ValType(this.Item(1)[nCol])
  Do Case
   Case vValType=='C'
    If lcDate(this.Item(1)[nCol])
     Aadd(aValType,'dTos(cTod(uTempData))')
    Else
     Aadd(aValType,'uTempData')
    Endif
   Case vValType=='D'
    Aadd(aValType,'dTos(uTempData)')
   Case vValType=='N'
    Aadd(aValType,'Str(uTempData)')
   Case vValType=='L'
    Aadd(aValType,'if(uTempData,".T.",".F.")')
   OtherWise
    // Tipo de dato no soportado
    // PENDIENTE: EXCLUSION DE CAMPOS DE TIPOS NO SOPORTADOS
    //            EN CONTENIDO A ORDENAR
    RETURN( .F. )
  EndCase
 Next

 // Generacin de contenidos a ordenar
 if lTest
  Aadd(aTest,'Generacin de contenidos a ordenar ' + Time())
 endif

 For nRow := 1 To nRowGrid
  // Almacenamiento temporal de datos originales del GRID
  Aadd(aDataGrid,aClone(this.Item(nRow)))
  // Se asigna el contenido de la columna requerida
  uTempData := this.Item(nRow)[_nGridCol]
  // Se efecta la conversin de tipo de datos para iniciar la cadena a ser ordenada
  cContOrd := &(aValType[_nGridCol]) + cFieldSeparator
  For nCol := 1 To nColProces
   // Se asigna el contenido de las columnas restantes
   if nCol<>_nGridCol
    uTempData := this.Item(nRow)[nCol]
    // Se efecta la conversin de tipo de datos para actualizar la cadena a ser ordenada
    cContOrd  := cContOrd + &(aValType[nCol]) + cFieldSeparator
   endif
  Next
  // Se integra el nmero de item para la recuperacin de los datos originales
  cContOrd := cContOrd + cFieldSeparator + cItemSeparator + Str(nRow)
  Aadd(aGridIndex,cContOrd)
 Next

 // Ordenamiento del contenido
 if lTest
  Aadd(aTest,'Ordenamiento del contenido ' + Time())
 endif

 // PENDIENTE: ASCENDENTE/DESCENDENTE
 aGridIndex := Asort(aGridIndex)


 // Transferencia del contenido ordenado
 if lTest
  Aadd(aTest,'Transferencia del contenido ordenado ' + Time())
 endif

 For nRow := 1 To nRowGrid

  // Se obtiene el contenido ordenado
  cContOrd := aGridIndex[nRow]
  // Se obtiene el nmero de item de origen del contenido ordenado
  nItemOrd := Val(SubStr(cContOrd,At(cFieldSeparator+cItemSeparator,cContOrd) + ;
                  Len(cFieldSeparator)+Len(cItemSeparator)))
  // Se verifica la existencia de errores
  if nItemOrd = 0
   lTransferError := .T. 
   LOOP
  endif

  // Se transfiere el contenido ordenado al control GRID
  this.Item(nRow) := Aclone(aClone(aDataGrid[nItemOrd]))

 Next

 // Recuperacin de datos originales por error en transferencia
 if lTransferError
  MsgStop("Error de transferencia interna de contenidos.","GRIDSETORDER()")
  // Transferencia del contenido original
  For nRow := 1 To nRowGrid
   // Se transfiere el contenido original al control GRID
   this.Item(nRow) := aClone(aDataGrid[nRow])
  Next
 endif

 if lTest
  MsgInfo(aTest[1]+Chr(13)+aTest[2]+Chr(13)+aTest[3]+Chr(13)+aTest[4]+Chr(13)+"Fin "+Time())
 endif

 // Actualizacin de interfaz
 this.Refresh()

RETURN(.T.)

//--------------------------------------------------------------//
