/*
              Fecha: 20/05/09
              Autor: Adrin Sergio Buratovich
        Descripcin: Funcionalidad complementaria para procesamiento
                     de cadenas de texto
           Objetivo: Complementar las funciones existentes en el lenguaje
                     hasta encontrar funciones nativas ya existentes
    Ultima Revisin: 20/05/09 - 17:00:00

  NOTA 01: Se incluye la funcin cStrToken() en reemplazo del
           uso de strtoken() ya que sta ltima produce errores
           en ciertos casos.
           EJEMPLO:

           StrToken('|2|3|4',1,'|') DEVUELVE '2'
           cStrToken('|2|3|4',1,'|') DEVUELVE ''

  NOTA 02: Se incluye la funcin lcDate() para la determinacin 
           de cadenas con formato de fecha

           Esta funcin slo soporta fechas en formatos BRITISH /
           FRENCH / JAPAN / AMERICAN 

           PENDIENTE: PARAMETRIZACION DEL FORMATO VALIDADO 
                      SEGUN EL ESTADO "SET DATE"


*/

//--------------------------------------------------------------//

FUNCTION cStrToken( _cCadena, _nPosDato, _cSeparador )

/*

 REEMPLAZA EL USO DE STRTOKEN YA QUE PRODUCE ERRORES
 CUANDO EXISTEN POSICIONES INICIALES SIN DATOS

 EJEMPLO:

  StrToken('|2|3|4',1,'|') DEVUELVE '2'
 cStrToken('|2|3|4',1,'|') DEVUELVE ''

*/

*------------------------
 Local nByte            := 0
 Local nBytes           := Len(_cCadena)
 Local nSeparador       := 0
*------------------------
 Local cByte            := ""
 Local cDato            := ""
*------------------------

 DEFAULT _cSeparador := '|'
 DEFAULT _nPosDato   := 1

 For nByte := 1 To nBytes

  cByte  := SubStr(_cCadena,nByte,1)

  If cByte == _cSeparador
   nSeparador++
   if nSeparador == _nPosDato
     RETURN( cDato )
   else
    cByte := ""
    cDato := ""
   endif
  EndIf

  cDato := cDato + cByte

 Next

 If nSeparador < ( _nPosDato - 1 )
  cDato := ""
 EndIf

RETURN( cDato )

//----------------------------------------------------------------------//

FUNCTION lcDate( _cCadena )

/*

 DETERMINA SI UNA CADENA DE CARACTERES POSEE UN FORMATO DE FECHA

 Esta funcin slo soporta fechas en formatos BRITISH / FRENCH / JAPAN / AMERICAN

 PENDIENTE: PARAMETRIZACION DEL FORMATO VALIDADO SEGUN EL ESTADO "SET DATE"

*/

*------------------------
 Local nByte            := 0
 Local nBytes           := Len(_cCadena)
 Local nSeparador       := 0
 Local cSeparador       := '/'
*------------------------
 Local cByte            := ""
 Local cDigitos         := " 0123456789"
 Local lDigitos         := .T.
*------------------------

 // Procesamiento
 For nByte := 1 To nBytes
  cByte  := SubStr(_cCadena,nByte,1)
  If cByte == cSeparador
   nSeparador++
  Else
   if !( cByte $ cDigitos )
    lDigitos := .F.
   endif
  EndIf
 Next

 // Validaciones

 // Deben existir 2 barras separadoras
 If nSeparador <> 2
  RETURN( .F. )
 EndIf
 // El tamao de la cadena debe contener entre 8 ( DD/MM/AAAA ) y 10 ( DD/MM/AAAA ) caracteres
 If !( nBytes>7 .and. nBytes<11 )
  RETURN( .F. )
 EndIf
 // Los caracteres entre separadores deben ser numricos o vacios
 If !lDigitos
  RETURN( .F. )
 EndIf

RETURN( .T. )

//----------------------------------------------------------------------//
